/*******************************************************************************
* Copyright 2020-2021 Intel Corporation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*******************************************************************************/

#pragma once

#include "oneapi/dal/detail/common.hpp"

namespace oneapi::dal::preview::csv::detail {

ONEDAL_EXPORT std::int32_t daal_string_to_int(const char *nptr, char **endptr);

ONEDAL_EXPORT double daal_string_to_double(const char *nptr, char **endptr);

template <typename T>
inline T daal_string_to(const char *nptr, char **endptr);

template <>
inline std::int32_t daal_string_to(const char *nptr, char **endptr) {
    return daal_string_to_int(nptr, endptr);
}

template <>
inline double daal_string_to(const char *nptr, char **endptr) {
    return daal_string_to_double(nptr, endptr);
}

} // namespace oneapi::dal::preview::csv::detail
