/* file: linear_model_training_batch.h */
/*******************************************************************************
* Copyright 2014-2021 Intel Corporation
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*******************************************************************************/

/*
//++
//  Implementation of the interface for the regression model-based training
//  in the batch processing mode
//--
*/

#ifndef __LINEAR_MODEL_TRAINING_BATCH_H__
#define __LINEAR_MODEL_TRAINING_BATCH_H__

#include "algorithms/linear_model/linear_model_training_types.h"
#include "algorithms/regression/regression_training_batch.h"

namespace daal
{
namespace algorithms
{
namespace linear_model
{
namespace training
{
namespace interface1
{
/**
 * @defgroup linear_model_training_batch Batch
 * @ingroup linear_model_training
 * @{
 */
/**
 * <a name="DAAL-CLASS-ALGORITHMS__LINEAR_MODEL__TRAINING__BATCH"></a>
 * \brief Provides methods for linear model model-based training in the batch processing mode
 *
 * \par References
 *      - \ref linear_model::interface1::Model "linear_model::Model" class
 *      - \ref prediction::interface1::Batch "prediction::Batch" class
 */
class DAAL_EXPORT Batch : public regression::training::Batch
{
public:
    typedef algorithms::linear_model::training::Input InputType;
    typedef algorithms::linear_model::Parameter ParameterType;
    typedef algorithms::linear_model::training::Result ResultType;

    /**
     * Returns the structure that contains the result of linear model model-based training
     * \return Structure that contains the result of linear model model-based training
     */
    ResultPtr getResult() { return ResultType::cast(_result); }
};
/** @} */
} // namespace interface1
using interface1::Batch;
} // namespace training
} // namespace linear_model
} // namespace algorithms
} // namespace daal
#endif
